///////////////////////////////////////////////////////////////////////////
// Filename:    Do178Builder.h
// Purpose:     Header for the Do178Builder project.
// Mods:        09/23/01 RSB    Began.  This file was initially created
//                              by wxDesigner, and is modified by wxDesigner
//                              every time the user interface is tweaked.
//                              However, a lot of other stuff has also been
//                              added to it manually, since all of the 
//                              Do178Builder files (and not just the ones
//                              related to wxWindows) include this file.
//                              The program comments in this file beginning
//                              "WDR:" are used by wxDesigner, and should
//                              not be modified.
//              10/21/01 RSB    Added control of output directory.
//      02/18/02 RSB    Added OnSize to the XmlEditorDialogClass.

/*
  Copyright 2001,2002 Ronald S. Burkey

  This file is part of Do178Builder.

  Do178Builder is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Do178Builder is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Do178Builder; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/////////////////////////////////////////////////////////////////////////////

#ifndef __Do178Builder_H__
#define __Do178Builder_H__

#ifdef __GNUG__
#pragma interface "Do178Builder.cpp"
#endif

// Include wxWindows' headers

#ifndef WX_PRECOMP
#include <wx/wx.h>
#endif

#include "Do178Builder_wdr.h"

// Helpful constants for the criticality levels and control categories.
#define DO178A_LEVEL_1 0
#define DO178A_LEVEL_2 1
#define DO178A_LEVEL_3 2
#define DO178B_LEVEL_A 3
#define DO178B_LEVEL_B 4
#define DO178B_LEVEL_C 5
#define DO178B_LEVEL_D 6
#define DO178B_LEVEL_E 7
#define NUM_CRITICALITY_LEVELS 8
#define SATISFIED_DISCRETION 0x00
#define SATISFIED_YES 0x01
#define SATISFIED_INDEPENDENCE 0x02
#define SATISFIED_MASK 0x0f
#define CC_DISCRETION 0x00
#define CC_2 0x10
#define CC_1 0x20
#define CC_MASK 0xf0

// Other constants, for the document types:
#define PSAC 0
#define SDP 1
#define SVP 2
#define SCMP 3
#define SQAP 4
#define SRS 5
#define SDS 6
#define SCS 7
#define SRD 8
#define DD 9
#define SDD 9
#define SVCP 10
#define SECI 11
#define SCI 12
#define SAS 13
#define NUM_DOCUMENTS 14
#define DOCUMENT_DISCRETION 0
#define DOCUMENT_YES 1

//----------------------------------------------------------------------------

// This is the data structure for a single XML record.  The entire XML
// table is a bunch of these linked together.  It's important to not somewhere,
// so we do it here:  Although DO-178B is a complete rewrite of DO-178A, 
// we're hoping to jam the various DO-178A documents into the same 
// document types (PSAC, SAS, etc.) and control types (CC1, CC2) as DO-178B.
struct XmlStruct
{
  // Links.
  XmlStruct *Next;
  XmlStruct *Children;
  XmlStruct *Parent;
  // Data.  
  char *Name;           // Link.
  char *Type;           // Link.
  char *Hint;           // Link.
  char *Text;           // Link.
  // The following fields relate the data contained in this item to the
  // various DO-178B documents.  Use with the constants defined above, like so:
  //    Documents[SDP] = DOCUMENT_YES;
  int Documents[NUM_DOCUMENTS];
  char Sections[NUM_DOCUMENTS][64];
  int SectionLevel[NUM_DOCUMENTS];
  // Here's where the control types for the records are indicated, 
  // vs. criticality level.  Use the constants defined above, like so:
  //    Applicability[DO178B_LEVEL_C] = APPLICAILITY_YES | CC_2;
  int Applicability[NUM_CRITICALITY_LEVELS];
  // Here's the wxTreeItemId given to this record in the displayed wxTreeCtrl.
  wxTreeItemId Id;
  wxColour *Color;      // Color of the item's text.
};

int LoadXmlFile (const char *Filename, struct XmlStruct *Xml);
int SaveXmlFile (const char *Filename, struct XmlStruct *Xml);
void ClearXml (struct XmlStruct *Xml);
struct XmlStruct *FindXml (struct XmlStruct *Root, wxTreeItemId Index);
int XmlToSgml (struct XmlStruct *Root, const char *OutputDirectory);

// WDR: class declarations

//----------------------------------------------------------------------------
// SearchBoxClass
//----------------------------------------------------------------------------

class SearchBoxClass:public wxDialog
{
public:
  // constructors and destructors
  SearchBoxClass (wxWindow * parent, wxWindowID id, const wxString & title,
          const wxPoint & pos = wxDefaultPosition,
          const wxSize & size = wxDefaultSize,
          long style = wxDEFAULT_DIALOG_STYLE);

  // WDR: method declarations for SearchBoxClass

private:
  // WDR: member variable declarations for SearchBoxClass

private:
  // WDR: handler declarations for SearchBoxClass
  void CheckSearchSas (wxCommandEvent & event);
  void CheckSearchSci (wxCommandEvent & event);
  void CheckSearchSeci (wxCommandEvent & event);
  void CheckSearchSvcp (wxCommandEvent & event);
  void CheckSearchSdd (wxCommandEvent & event);
  void CheckSearchSrd (wxCommandEvent & event);
  void CheckSearchScs (wxCommandEvent & event);
  void CheckSearchSds (wxCommandEvent & event);
  void CheckSearchSrs (wxCommandEvent & event);
  void CheckSearchSqap (wxCommandEvent & event);
  void CheckSearchScmp (wxCommandEvent & event);
  void CheckSearchSvp (wxCommandEvent & event);
  void CheckSearchSdp (wxCommandEvent & event);
  void CheckSearchPsac (wxCommandEvent & event);
  void CheckAllDocs (wxCommandEvent & event);
  void ClickSearchCancel (wxCommandEvent & event);
  void ClickSearchButton (wxCommandEvent & event);
  void CheckSearchHints (wxCommandEvent & event);
  void CheckSearchText (wxCommandEvent & event);
  void CheckSearchTitles (wxCommandEvent & event);

private:
  DECLARE_EVENT_TABLE ()};

//----------------------------------------------------------------------------
// ProjectOptionEditorClass
//----------------------------------------------------------------------------

class ProjectOptionEditorClass:public wxDialog
{
public:
  // constructors and destructors
  ProjectOptionEditorClass (wxWindow * parent, wxWindowID id,
                const wxString & title, const wxPoint & pos =
                wxDefaultPosition, const wxSize & size =
                wxDefaultSize, long style =
                wxDEFAULT_DIALOG_STYLE);

  // WDR: method declarations for ProjectOptionEditorClass

private:
  // WDR: member variable declarations for ProjectOptionEditorClass

private:
  // WDR: handler declarations for ProjectOptionEditorClass
  //void DoneXml (wxCommandEvent & event);
  void CancelProjectOptions (wxCommandEvent & event);
  void OkProjectOptions (wxCommandEvent & event);
  void ProjectOptions (wxMenuEvent & event);

private:
  DECLARE_EVENT_TABLE ()};

//----------------------------------------------------------------------------
// XmlEditorDialogClass
//----------------------------------------------------------------------------

class XmlEditorDialogClass:public wxDialog
{
public:
  // constructors and destructors
  XmlEditorDialogClass (wxWindow * parent, wxWindowID id,
            const wxString & title, const wxPoint & pos =
            wxDefaultPosition, const wxSize & size =
            wxDefaultSize, long style = wxDEFAULT_DIALOG_STYLE);

  // WDR: method declarations for XmlEditorDialogClass
    wxRadioButton* GetExpertbutton()  { return (wxRadioButton*) FindWindow( ID_EXPERTBUTTON ); }
    wxRadioButton* GetNovicebutton()  { return (wxRadioButton*) FindWindow( ID_NOVICEBUTTON ); }
    wxBitmapButton* GetMarkupbutton()  { return (wxBitmapButton*) FindWindow( ID_MARKUPBUTTON ); }
    wxChoice* GetMarkupchoice()  { return (wxChoice*) FindWindow( ID_MARKUPCHOICE ); }

public:
  void FindAgainButton (wxCommandEvent & event);
  void ClickCleanup (struct XmlStruct *Record);
  struct XmlStruct *RootXml, *Template;
  int ClickTreeBypass;
  wxTreeItemId LastSelected;
  wxTreeCtrl *ItemTree;
  void OnClose (wxCommandEvent & event);
  void OnSize(wxSizeEvent& event);
  void XmlChange (wxTreeItemId Selected, struct XmlStruct *Xml);
  void FrameworkSet (struct XmlStruct *Xml);
  int DatasetNewlyChanged;
  void DocumentChange (struct XmlStruct *, int DocNum, wxCheckBox * Box);
  void SatChange (struct XmlStruct *, int Level, wxBitmapButton * Button);
  void CcChange (struct XmlStruct *, int Level, wxBitmapButton * Button);
  void ButtonBitmap (wxBitmapButton * Button, wxBitmap Bitmap);
  void SetSectionDocs (struct XmlStruct *Record);
  struct XmlStruct **FindPredecessor (struct XmlStruct *Root,
                      struct XmlStruct *Xml);
  void ClickTree (wxTreeEvent & event);
  // Stuff related to search parameters.
  wxString SearchString;
  bool SearchDocs[NUM_DOCUMENTS];
  bool SearchAll, SearchTitles, SearchHints, SearchText;
  wxTreeItemId LastSearchPos;

private:
  // WDR: member variable declarations for XmlEditorDialogClass

  // The color used to mark data items changed.
    wxColour MY_CHANGE_COLOR;
  wxColour *CHANGE_COLOR;


// These are pointers to various interesting widgets within an XML-editor
// dialog.  they are valid only while the dialog exists, and must be 
// regenerated each time the dialog starts.
  wxTextCtrl *HintBox, *ItemText;
  wxCheckBox *PsacCheck, *SdpCheck, *SvpCheck, *ScmpCheck, *SqapCheck,
    *SrsCheck, *SdsCheck, *ScsCheck, *SrdCheck, *DdCheck,
    *SvcpCheck, *SeciCheck, *SciCheck, *SasCheck;
  wxCheckBox *PsacSection, *SdpSection, *SvpSection, *ScmpSection,
    *SqapSection, *SrsSection, *SdsSection, *ScsSection, *SrdSection,
    *DdSection, *SvcpSection, *SeciSection, *SciSection, *SasSection;
  wxBitmapButton *BitmapSatA, *BitmapSatB, *BitmapSatC, *BitmapSatD,
    *BitmapSatE, *CcA, *CcB, *CcC, *CcD, *CcE;
  wxButton *Add, *AddSub, *Del;
  wxChoice *MarkupChoice;
  wxBitmapButton *MarkupButton;
  wxRadioButton *NoviceButton, *ExpertButton;

private:
  // WDR: handler declarations for XmlEditorDialogClass
  void ExpertCheck (wxCommandEvent & event);
  void NoviceCheck (wxCommandEvent & event);
  void MarkupButtonClick (wxCommandEvent & event);
  void FindButton (wxCommandEvent & event);
  struct XmlStruct *SeekTemplate (struct XmlStruct *Record);
  struct XmlStruct *CreateTreeCopy (struct XmlStruct *Xml);
  void FillSubTree (struct XmlStruct *Xml);
  struct XmlStruct *AllowableAddSiblings (struct XmlStruct *Record);
  struct XmlStruct *AllowableAddSubs (struct XmlStruct *Record);
  void SqarClick (wxCommandEvent & event);
  void ScmrClick (wxCommandEvent & event);
  void SvrClick (wxCommandEvent & event);
  void SaveButton (wxCommandEvent & event);
  void DeleteItem (wxCommandEvent & event);
  void AddSubItem (wxCommandEvent & event);
  void AddItem (wxCommandEvent & event);
  void ClickCcE (wxCommandEvent & event);
  void ClickCcD (wxCommandEvent & event);
  void ClickCcC (wxCommandEvent & event);
  void ClickCcB (wxCommandEvent & event);
  void ClickCcA (wxCommandEvent & event);
  void ClickSatE (wxCommandEvent & event);
  void ClickSatD (wxCommandEvent & event);
  void ClickSatC (wxCommandEvent & event);
  void ClickSatB (wxCommandEvent & event);
  void ClickSatA (wxCommandEvent & event);
  void SasClick (wxCommandEvent & event);
  void SciClick (wxCommandEvent & event);
  void SeciClick (wxCommandEvent & event);
  void SvcpClick (wxCommandEvent & event);
  void DdClick (wxCommandEvent & event);
  void SrdClick (wxCommandEvent & event);
  void ScsClick (wxCommandEvent & event);
  void SdsClick (wxCommandEvent & event);
  void SrsClick (wxCommandEvent & event);
  void SqapClick (wxCommandEvent & event);
  void ScmpClick (wxCommandEvent & event);
  void SvpClick (wxCommandEvent & event);
  void SdpClick (wxCommandEvent & event);
  void PsacClick (wxCommandEvent & event);
  //void DoneXml (wxCommandEvent & event);

private:
  DECLARE_EVENT_TABLE ()};

//----------------------------------------------------------------------------
// Do178BuilderFrame
//----------------------------------------------------------------------------

class Do178BuilderFrame:public wxFrame
{
public:
  // constructors and destructors
  Do178BuilderFrame (wxWindow * parent, wxWindowID id, const wxString & title,
             const wxPoint & pos = wxDefaultPosition,
             const wxSize & size = wxDefaultSize,
             long style = wxDEFAULT_FRAME_STYLE);

public:
  int FileIsOpen;
  void OpenTemplate (void);

private:
  // WDR: method declarations for Do178BuilderFrame
  void CreateMyMenuBar ();
  void CreateMyToolBar ();
  void LoadXmlToEditor (struct XmlStruct *, XmlEditorDialogClass &,
            wxTreeItemId);

private:
  // WDR: member variable declarations for Do178BuilderFrame

private:
  // WDR: handler declarations for Do178BuilderFrame
  void BuildDocuments (wxMenuEvent & event);
  void SaveFileAs (wxMenuEvent & event);
  void SaveFile (wxMenuEvent & event);
  void CloseFile (wxMenuEvent & event);
  void HintEditCheck (wxMenuEvent & event);
  void EditDataset (wxMenuEvent & event);
  void ProjectOptions (wxMenuEvent & event);
  void Open (wxMenuEvent & event);
  //void OpenMenu( wxCommandEvent &event );
  void OnAbout (wxCommandEvent & event);
  void OnQuit (wxCommandEvent & event);
  void OnCloseWindow (wxCloseEvent & event);

private:
  int UnsavedCancel (void);

private:
  DECLARE_EVENT_TABLE ()};

//----------------------------------------------------------------------------
// Do178BuilderApp
//----------------------------------------------------------------------------

class Do178BuilderApp:public wxApp
{
public:
  Do178BuilderApp ();

  virtual bool OnInit ();
  virtual int OnExit ();
};

#endif
