/////////////////////////////////////////////////////////////////////////////
// Name:        SearchBoxClass.cpp
// Author:      Ron Burkey
// Purpose:     Event handlers for the search-setup box of Do178Builder.
//              In other words, most of the code for the SearchBoxClass class.
// Mods:        10/21/01 RSB    Split off from Do178Builder.cpp.

/*
  Copyright 2001 Ronald S. Burkey

  This file is part of Do178Builder.

  Do178Builder is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Do178Builder is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Do178Builder; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/////////////////////////////////////////////////////////////////////////////

// For compilers that support precompilation
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

// Include private headers
#include "Do178Builder.h"

#include <sys/types.h>
#include <dirent.h>
#include <unistd.h>

//----------------------------------------------------------------------------
// Constructor

SearchBoxClass::SearchBoxClass (wxWindow * parent, wxWindowID id, const wxString & title, const wxPoint & position, const wxSize & size, long style):
wxDialog (parent, id, title, position, size, style)
{
  SearchBox (this, TRUE);
}

//---------------------------------------------------------------------------
// Varous things that happen when boxes in the dialog are checked.  They're
// all mostly the same, so I won't comment them separately.  Basically, 
// there is global storage for the data within the XmlEditorDialogClass class
// that's the parent of this dialog, and the actions of activating or 
// deactivating the controls is to modify this global data.  In retrospect 
// I could have avoide all that hassle by simply not creating and destroying
// the search dialog all the time -- but, live and learn, I guess.

void
SearchBoxClass::CheckSearchSas (wxCommandEvent & event)
{
  ((XmlEditorDialogClass *) GetParent ())->SearchDocs[SAS] =
    event.IsChecked ();
}

void
SearchBoxClass::CheckSearchSci (wxCommandEvent & event)
{
  ((XmlEditorDialogClass *) GetParent ())->SearchDocs[SCI] =
    event.IsChecked ();
}

void
SearchBoxClass::CheckSearchSeci (wxCommandEvent & event)
{
  ((XmlEditorDialogClass *) GetParent ())->SearchDocs[SECI] =
    event.IsChecked ();
}

void
SearchBoxClass::CheckSearchSvcp (wxCommandEvent & event)
{
  ((XmlEditorDialogClass *) GetParent ())->SearchDocs[SVCP] =
    event.IsChecked ();
}

void
SearchBoxClass::CheckSearchSdd (wxCommandEvent & event)
{
  ((XmlEditorDialogClass *) GetParent ())->SearchDocs[SDD] =
    event.IsChecked ();
}

void
SearchBoxClass::CheckSearchSrd (wxCommandEvent & event)
{
  ((XmlEditorDialogClass *) GetParent ())->SearchDocs[SRD] =
    event.IsChecked ();
}

void
SearchBoxClass::CheckSearchScs (wxCommandEvent & event)
{
  ((XmlEditorDialogClass *) GetParent ())->SearchDocs[SCS] =
    event.IsChecked ();
}

void
SearchBoxClass::CheckSearchSds (wxCommandEvent & event)
{
  ((XmlEditorDialogClass *) GetParent ())->SearchDocs[SDS] =
    event.IsChecked ();
}

void
SearchBoxClass::CheckSearchSrs (wxCommandEvent & event)
{
  ((XmlEditorDialogClass *) GetParent ())->SearchDocs[SRS] =
    event.IsChecked ();
}

void
SearchBoxClass::CheckSearchSqap (wxCommandEvent & event)
{
  ((XmlEditorDialogClass *) GetParent ())->SearchDocs[SQAP] =
    event.IsChecked ();
}

void
SearchBoxClass::CheckSearchScmp (wxCommandEvent & event)
{
  ((XmlEditorDialogClass *) GetParent ())->SearchDocs[SCMP] =
    event.IsChecked ();
}

void
SearchBoxClass::CheckSearchSvp (wxCommandEvent & event)
{
  ((XmlEditorDialogClass *) GetParent ())->SearchDocs[SVP] =
    event.IsChecked ();
}

void
SearchBoxClass::CheckSearchSdp (wxCommandEvent & event)
{
  ((XmlEditorDialogClass *) GetParent ())->SearchDocs[SDP] =
    event.IsChecked ();
}

void
SearchBoxClass::CheckSearchPsac (wxCommandEvent & event)
{
  ((XmlEditorDialogClass *) GetParent ())->SearchDocs[PSAC] =
    event.IsChecked ();
}

void
SearchBoxClass::CheckAllDocs (wxCommandEvent & event)
{
  bool Checked;
  wxWindow *dialog = GetParent ();
  Checked = event.IsChecked ();
  ((XmlEditorDialogClass *) GetParent ())->SearchAll = Checked;
  wxDynamicCast (dialog->FindWindow (ID_SEARCHPSAC),
		 wxCheckBox)->Enable (!Checked);
  wxDynamicCast (dialog->FindWindow (ID_SEARCHSDP),
		 wxCheckBox)->Enable (!Checked);
  wxDynamicCast (dialog->FindWindow (ID_SEARCHSVP),
		 wxCheckBox)->Enable (!Checked);
  wxDynamicCast (dialog->FindWindow (ID_SEARCHSCMP),
		 wxCheckBox)->Enable (!Checked);
  wxDynamicCast (dialog->FindWindow (ID_SEARCHSQAP),
		 wxCheckBox)->Enable (!Checked);
  wxDynamicCast (dialog->FindWindow (ID_SEARCHSRS),
		 wxCheckBox)->Enable (!Checked);
  wxDynamicCast (dialog->FindWindow (ID_SEARCHSDS),
		 wxCheckBox)->Enable (!Checked);
  wxDynamicCast (dialog->FindWindow (ID_SEARCHSCS),
		 wxCheckBox)->Enable (!Checked);
  wxDynamicCast (dialog->FindWindow (ID_SEARCHSRD),
		 wxCheckBox)->Enable (!Checked);
  wxDynamicCast (dialog->FindWindow (ID_SEARCHSDD),
		 wxCheckBox)->Enable (!Checked);
  wxDynamicCast (dialog->FindWindow (ID_SEARCHSVCP),
		 wxCheckBox)->Enable (!Checked);
  wxDynamicCast (dialog->FindWindow (ID_SEARCHSECI),
		 wxCheckBox)->Enable (!Checked);
  wxDynamicCast (dialog->FindWindow (ID_SEARCHSCI),
		 wxCheckBox)->Enable (!Checked);
  wxDynamicCast (dialog->FindWindow (ID_SEARCHSAS),
		 wxCheckBox)->Enable (!Checked);
}

void
SearchBoxClass::ClickSearchCancel (wxCommandEvent & event)
{
  EndModal (0);
}

void
SearchBoxClass::ClickSearchButton (wxCommandEvent & event)
{
  ((XmlEditorDialogClass *) GetParent ())->SearchString =
    wxDynamicCast (FindWindow (ID_SEARCHBOX), wxTextCtrl)->GetValue ();
  ((XmlEditorDialogClass *) GetParent ())->FindAgainButton (event);
  EndModal (1);
}

void
SearchBoxClass::CheckSearchHints (wxCommandEvent & event)
{
  ((XmlEditorDialogClass *) GetParent ())->SearchHints = event.IsChecked ();
}

void
SearchBoxClass::CheckSearchText (wxCommandEvent & event)
{
  ((XmlEditorDialogClass *) GetParent ())->SearchText = event.IsChecked ();
}

void
SearchBoxClass::CheckSearchTitles (wxCommandEvent & event)
{
  ((XmlEditorDialogClass *) GetParent ())->SearchTitles = event.IsChecked ();
}
