///////////////////////////////////////////////////////////////////////////
// Filename:    XmlEditorDialogClass.cpp
// Author:      Ron Burkey
// Purpose:     The code for the XmlEditorDialogClass C++ class.
// Mods:        10/03/01 RSB    Split off from Do178Builder.cpp.
//              10/21/01 RSB    Added code for the find and find-again
//                              buttons.
//              10/28/01 RSB    Began adding stuff for italics, bold, underlining,
//                              and fixed-width. ... but it turns out that the
//                              rich-text stuff I thought had been implemented
//                              in wxWindows hasn't been, so none of it does
//                              anything.
//              10/31/01 RSB    Removed Italics, bold, underline, and code
//                              markup (which were incomplete and not working
//                              anyhow), in favor of the beginnings of a
//                              general DocBook markup feature.
//		02/18/02 RSB	Tried fixing the markup-insertion bug in Win32.
//				Made some attempt to fix the Win32 button
//				non-refresh problem.  Tried to fix the resizing
//				problem in Win32.

/*
  Copyright 2001,2002 Ronald S. Burkey

  This file is part of Do178Builder.

  Do178Builder is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Do178Builder is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Do178Builder; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/*
  The XmlEditorDialogClass implements the dialog in the Do178Builder
  program used for editing the data for a Do178Builder project.
  This is basically all of the code for the class, except as follows:
  
	Do178Builder.h		Contains the prototype for the class
				and for the XmlStruct structure, along
				with unrelated items.
	Do178Builder.cpp	Contains the wxWindows event table for
				the dialog, and items unrelated to 
				the class.
	Do178Builder_wdr.cpp	Contains functions for constructing 
				the dialog from wxWindows widgets,
				functions providing pointers to all
				relevant bitmaps, and various items
				unrelated to the class.
				
  The reason for this (perhaps) rather odd partitioning of the code is
  that the wxDesigner RAD environment was used to construct the user 
  interface, and it arranges the code in this way.  It also places all
  of the event handlers for the class into Do178Builder.cpp, but these
  have been moved to XmlEditorDialogClass.cpp manually.  
*/

/////////////////////////////////////////////////////////////////////////

// For compilers that support precompilation
#include "wx/wxprec.h"

// Include private headers
#include "Do178Builder.h"

#include <sys/types.h>
#include <dirent.h>
#include <unistd.h>
extern bool HintEditable;
extern int DatasetChanged;
extern wxString rFilename;

//---------------------------------------------------------------------------
// The constructor for the class.

XmlEditorDialogClass::XmlEditorDialogClass (wxWindow *
					    parent,
					    wxWindowID id,
					    const wxString &
					    title,
					    const wxPoint &
					    position,
					    const wxSize & size, long style):
wxDialog (parent, id, title, position, size, style)
{
  int i;
  XmlEditorDialog (this, TRUE);

  // The color used to mark data items changed.
  MY_CHANGE_COLOR = wxColour (0, 0, 255);
  CHANGE_COLOR = &MY_CHANGE_COLOR;

  // Regenerate pointers to all of the dialog elements we need to address directly.
  MarkupChoice = GetMarkupchoice ();
  MarkupButton = GetMarkupbutton ();
  NoviceButton = GetNovicebutton ();
  ExpertButton = GetExpertbutton ();
  ItemTree = wxDynamicCast (FindWindow ("ItemTree"), wxTreeCtrl);
  HintBox = wxDynamicCast (FindWindow ("HintBox"), wxTextCtrl);
  ItemText = wxDynamicCast (FindWindow ("ItemText"), wxTextCtrl);
  PsacCheck = wxDynamicCast (FindWindow ("PsacCheck"), wxCheckBox);
  SdpCheck = wxDynamicCast (FindWindow ("SdpCheck"), wxCheckBox);
  SvpCheck = wxDynamicCast (FindWindow ("SvpCheck"), wxCheckBox);
  ScmpCheck = wxDynamicCast (FindWindow ("ScmpCheck"), wxCheckBox);
  SqapCheck = wxDynamicCast (FindWindow ("SqapCheck"), wxCheckBox);
  SrsCheck = wxDynamicCast (FindWindow ("SrsCheck"), wxCheckBox);
  SdsCheck = wxDynamicCast (FindWindow ("SdsCheck"), wxCheckBox);
  ScsCheck = wxDynamicCast (FindWindow ("ScsCheck"), wxCheckBox);
  SrdCheck = wxDynamicCast (FindWindow ("SrdCheck"), wxCheckBox);
  DdCheck = wxDynamicCast (FindWindow ("DdCheck"), wxCheckBox);
  SvcpCheck = wxDynamicCast (FindWindow ("SvcpCheck"), wxCheckBox);
  SeciCheck = wxDynamicCast (FindWindow ("SeciCheck"), wxCheckBox);
  SciCheck = wxDynamicCast (FindWindow ("SciCheck"), wxCheckBox);
  SasCheck = wxDynamicCast (FindWindow ("SasCheck"), wxCheckBox);
  PsacSection = wxDynamicCast (FindWindow ("PsacSection"), wxCheckBox);
  SdpSection = wxDynamicCast (FindWindow ("SdpSection"), wxCheckBox);
  SvpSection = wxDynamicCast (FindWindow ("SvpSection"), wxCheckBox);
  ScmpSection = wxDynamicCast (FindWindow ("ScmpSection"), wxCheckBox);
  SqapSection = wxDynamicCast (FindWindow ("SqapSection"), wxCheckBox);
  SrsSection = wxDynamicCast (FindWindow ("SrsSection"), wxCheckBox);
  SdsSection = wxDynamicCast (FindWindow ("SdsSection"), wxCheckBox);
  ScsSection = wxDynamicCast (FindWindow ("ScsSection"), wxCheckBox);
  SrdSection = wxDynamicCast (FindWindow ("SrdSection"), wxCheckBox);
  DdSection = wxDynamicCast (FindWindow ("DdSection"), wxCheckBox);
  SvcpSection = wxDynamicCast (FindWindow ("SvcpSection"), wxCheckBox);
  SeciSection = wxDynamicCast (FindWindow ("SeciSection"), wxCheckBox);
  SciSection = wxDynamicCast (FindWindow ("SciSection"), wxCheckBox);
  SasSection = wxDynamicCast (FindWindow ("SasSection"), wxCheckBox);
  BitmapSatA = wxDynamicCast (FindWindow ("BitmapSatA"), wxBitmapButton);
  BitmapSatB = wxDynamicCast (FindWindow ("BitmapSatB"), wxBitmapButton);
  BitmapSatC = wxDynamicCast (FindWindow ("BitmapSatC"), wxBitmapButton);
  BitmapSatD = wxDynamicCast (FindWindow ("BitmapSatD"), wxBitmapButton);
  BitmapSatE = wxDynamicCast (FindWindow ("BitmapSatE"), wxBitmapButton);
  CcA = wxDynamicCast (FindWindow ("CcA"), wxBitmapButton);
  CcB = wxDynamicCast (FindWindow ("CcB"), wxBitmapButton);
  CcC = wxDynamicCast (FindWindow ("CcC"), wxBitmapButton);
  CcD = wxDynamicCast (FindWindow ("CcD"), wxBitmapButton);
  CcE = wxDynamicCast (FindWindow ("CcE"), wxBitmapButton);
  Add = wxDynamicCast (FindWindow ("Add"), wxButton);
  AddSub = wxDynamicCast (FindWindow ("AddSub"), wxButton);
  Del = wxDynamicCast (FindWindow ("Del"), wxButton);

  // Enable the checkboxes, if appropriate.
  PsacCheck->Enable (HintEditable);
  SdpCheck->Enable (HintEditable);
  SvpCheck->Enable (HintEditable);
  ScmpCheck->Enable (HintEditable);
  SqapCheck->Enable (HintEditable);
  SrsCheck->Enable (HintEditable);
  SdsCheck->Enable (HintEditable);
  ScsCheck->Enable (HintEditable);
  SrdCheck->Enable (HintEditable);
  DdCheck->Enable (HintEditable);
  SvcpCheck->Enable (HintEditable);
  SeciCheck->Enable (HintEditable);
  SciCheck->Enable (HintEditable);
  SasCheck->Enable (HintEditable);
  BitmapSatA->Enable (HintEditable);
  BitmapSatB->Enable (HintEditable);
  BitmapSatC->Enable (HintEditable);
  BitmapSatD->Enable (HintEditable);
  BitmapSatE->Enable (HintEditable);
  CcA->Enable (HintEditable);
  CcB->Enable (HintEditable);
  CcC->Enable (HintEditable);
  CcD->Enable (HintEditable);
  CcE->Enable (HintEditable);

  // Default search parameters.
  SearchString = "";
  for (i = 0; i < NUM_DOCUMENTS; SearchDocs[i++] = 0);
  SearchAll = 1;
  SearchTitles = 1;
  SearchHints = 1;
  SearchText = 1;
  LastSearchPos = -1;

  // Miscellaneous stuff.
  HintBox->SetEditable (HintEditable);
  wxColour HintBackgroundColor;
  if (HintEditable)
    HintBackgroundColor = *wxWHITE;
  else
    HintBackgroundColor = *wxLIGHT_GREY;
  HintBox->SetBackgroundColour (HintBackgroundColor);
  LastSelected = -1;
  DatasetNewlyChanged = 0;
  ClickTreeBypass = 0;
  wxCommandEvent DummyEvent;
  NoviceCheck (DummyEvent);
}

//-------------------------------------------------------------
// Handlers for the Novice and Expert buttons.

extern const char *InlineTags[], *NoviceTags[];
extern int NumAllInlines, NumNovices;

void
XmlEditorDialogClass::ExpertCheck (wxCommandEvent & event)
{
  int i;
  wxChoice *MarkupChoice =
    wxDynamicCast (FindWindow (ID_MARKUPCHOICE), wxChoice);
  MarkupChoice->Clear ();
  for (i = 0; i < NumAllInlines; i++)
    MarkupChoice->Append (InlineTags[i], (void *) NULL);
  MarkupChoice->SetColumns (3);
}

void
XmlEditorDialogClass::NoviceCheck (wxCommandEvent & event)
{
  int i;
  wxChoice *MarkupChoice =
    wxDynamicCast (FindWindow (ID_MARKUPCHOICE), wxChoice);
  MarkupChoice->Clear ();
  for (i = 0; i < NumNovices; i++)
    MarkupChoice->Append (NoviceTags[i], (void *) NULL);
  MarkupChoice->SetColumns (1);
}

//-----------------------------------------------------------------
// Handler for the Markup button.

void
XmlEditorDialogClass::MarkupButtonClick (wxCommandEvent & event)
{
  long from, to, n;
  wxString Tag, s;
  ItemText->GetSelection (&from, &to);
  if (from == to)
    return;
  if (from > to)
    {
      n = from;
      from = to;
      to = n;
    }
  wxChoice *MarkupChoice =
    wxDynamicCast (FindWindow (ID_MARKUPCHOICE), wxChoice);
  Tag = MarkupChoice->GetStringSelection ();
  if (Tag)
    {
      s = "</" + Tag + ">";
      //ItemText->Replace (to, to, s);
      ItemText->SetInsertionPoint (to);
      ItemText->WriteText (s);
      s = "<" + Tag + ">";
      //ItemText->Replace (from, from, s);
      ItemText->SetInsertionPoint (from);
      ItemText->WriteText (s);
      ItemText->SetSelection (from + 2 + Tag.Length (),
			      to + 2 + Tag.Length ());
    }
}

//-----------------------------------------------------------------
// A handler that gets called when the "find again" button is 
// pressed.  It actually does all of the actual searching for
// the "find" button as well.

void
XmlEditorDialogClass::FindAgainButton (wxCommandEvent & event)
{
  wxMessageBox ("The search function is not implemented yet.",
		"So very sorry!", wxCENTRE | wxOK | wxICON_INFORMATION);
}

//-----------------------------------------------------------------
// A handler that gets called when the "find" button is pressed.

void
XmlEditorDialogClass::FindButton (wxCommandEvent & event)
{
  SearchBoxClass dialog (this, -1, "Define search parameters");
  wxDynamicCast (dialog.FindWindow (ID_SEARCHTITLES),
		 wxCheckBox)->SetValue (SearchTitles);
  wxDynamicCast (dialog.FindWindow (ID_SEARCHHINT),
		 wxCheckBox)->SetValue (SearchHints);
  wxDynamicCast (dialog.FindWindow (ID_SEARCHDESCRIPTIVE),
		 wxCheckBox)->SetValue (SearchText);
  wxDynamicCast (dialog.FindWindow (ID_SEARCHALL),
		 wxCheckBox)->SetValue (SearchAll);
  wxDynamicCast (dialog.FindWindow (ID_SEARCHPSAC),
		 wxCheckBox)->SetValue (SearchDocs[PSAC]);
  wxDynamicCast (dialog.FindWindow (ID_SEARCHSDP),
		 wxCheckBox)->SetValue (SearchDocs[SDP]);
  wxDynamicCast (dialog.FindWindow (ID_SEARCHSVP),
		 wxCheckBox)->SetValue (SearchDocs[SVP]);
  wxDynamicCast (dialog.FindWindow (ID_SEARCHSCMP),
		 wxCheckBox)->SetValue (SearchDocs[SCMP]);
  wxDynamicCast (dialog.FindWindow (ID_SEARCHSQAP),
		 wxCheckBox)->SetValue (SearchDocs[SQAP]);
  wxDynamicCast (dialog.FindWindow (ID_SEARCHSRS),
		 wxCheckBox)->SetValue (SearchDocs[SRS]);
  wxDynamicCast (dialog.FindWindow (ID_SEARCHSDS),
		 wxCheckBox)->SetValue (SearchDocs[SDS]);
  wxDynamicCast (dialog.FindWindow (ID_SEARCHSCS),
		 wxCheckBox)->SetValue (SearchDocs[SCS]);
  wxDynamicCast (dialog.FindWindow (ID_SEARCHSRD),
		 wxCheckBox)->SetValue (SearchDocs[SRD]);
  wxDynamicCast (dialog.FindWindow (ID_SEARCHSDD),
		 wxCheckBox)->SetValue (SearchDocs[SDD]);
  wxDynamicCast (dialog.FindWindow (ID_SEARCHSVCP),
		 wxCheckBox)->SetValue (SearchDocs[SVCP]);
  wxDynamicCast (dialog.FindWindow (ID_SEARCHSECI),
		 wxCheckBox)->SetValue (SearchDocs[SECI]);
  wxDynamicCast (dialog.FindWindow (ID_SEARCHSCI),
		 wxCheckBox)->SetValue (SearchDocs[SCI]);
  wxDynamicCast (dialog.FindWindow (ID_SEARCHSAS),
		 wxCheckBox)->SetValue (SearchDocs[SAS]);
  wxDynamicCast (dialog.FindWindow (ID_SEARCHPSAC),
		 wxCheckBox)->Enable (!SearchAll);
  wxDynamicCast (dialog.FindWindow (ID_SEARCHSDP),
		 wxCheckBox)->Enable (!SearchAll);
  wxDynamicCast (dialog.FindWindow (ID_SEARCHSVP),
		 wxCheckBox)->Enable (!SearchAll);
  wxDynamicCast (dialog.FindWindow (ID_SEARCHSCMP),
		 wxCheckBox)->Enable (!SearchAll);
  wxDynamicCast (dialog.FindWindow (ID_SEARCHSQAP),
		 wxCheckBox)->Enable (!SearchAll);
  wxDynamicCast (dialog.FindWindow (ID_SEARCHSRS),
		 wxCheckBox)->Enable (!SearchAll);
  wxDynamicCast (dialog.FindWindow (ID_SEARCHSDS),
		 wxCheckBox)->Enable (!SearchAll);
  wxDynamicCast (dialog.FindWindow (ID_SEARCHSCS),
		 wxCheckBox)->Enable (!SearchAll);
  wxDynamicCast (dialog.FindWindow (ID_SEARCHSRD),
		 wxCheckBox)->Enable (!SearchAll);
  wxDynamicCast (dialog.FindWindow (ID_SEARCHSDD),
		 wxCheckBox)->Enable (!SearchAll);
  wxDynamicCast (dialog.FindWindow (ID_SEARCHSVCP),
		 wxCheckBox)->Enable (!SearchAll);
  wxDynamicCast (dialog.FindWindow (ID_SEARCHSECI),
		 wxCheckBox)->Enable (!SearchAll);
  wxDynamicCast (dialog.FindWindow (ID_SEARCHSCI),
		 wxCheckBox)->Enable (!SearchAll);
  wxDynamicCast (dialog.FindWindow (ID_SEARCHSAS),
		 wxCheckBox)->Enable (!SearchAll);
  wxDynamicCast (dialog.FindWindow (ID_SEARCHBOX),
		 wxTextCtrl)->SetValue (SearchString);
  dialog.Center ();
  if (dialog.ShowModal ())
    wxDynamicCast (FindWindow (ID_FINDAGAINBUTTON),
		   wxBitmapButton)->Enable (true);
}

//---------------------------------------------------------------------------
// A handler that gets called when the dialog closes.
// We have to make sure that the in-memory XML structure gets updated
// with correct editor-box info, if the editor is closed.

void
XmlEditorDialogClass::OnClose (wxCommandEvent & event)
{
  XmlChange (ItemTree->GetSelection (), RootXml);
  EndModal (0);
}

//---------------------------------------------------------------------------
// This handler is called on resizing the dialog.  It shouldn't be necessary
// (and isn't in Linux), but in Win32 the toolbar stuff over the text
// control doesn't get redrawn properly.
// ... Oh, and it doesn't work anyhow.

void 
XmlEditorDialogClass::OnSize(wxSizeEvent& event)
{
  //MarkupChoice->Refresh ();
  //MarkupButton->Refresh ();
  //NoviceButton->Refresh ();
  //ExpertButton->Refresh ();
  //Refresh ();
}

//---------------------------------------------------------------------------
// A handler called when the "Save" button is clicked.

void
XmlEditorDialogClass::SaveButton (wxCommandEvent & event)
{
  // The purpose of the following lines is to force any data in the
  // text window to be saved.  It will basically mimic clicking
  // on the item with the mouse.
  wxTreeEvent DummyEvent;
  ClickTreeBypass = 1;
  ClickTree (DummyEvent);
  // Okay, save it.
  if (SaveXmlFile (rFilename, RootXml))
    wxMessageBox ("Could not save the file.", "Error",
		  wxCENTRE | wxOK | wxICON_ERROR);
  else
    {
      DatasetChanged = 0;
      wxMessageBox ("File saved.", "Status",
		    wxCENTRE | wxOK | wxICON_INFORMATION);
    }
}

//---------------------------------------------------------------------------
// A handler called when the "Delete" button is clicked.

void
XmlEditorDialogClass::DeleteItem (wxCommandEvent & event)
{
  wxTreeItemId Selected = ItemTree->GetSelection ();
  struct XmlStruct *Record = FindXml (RootXml, Selected);
  struct XmlStruct **Pred = FindPredecessor (RootXml, Record);
  if (Pred == NULL)
    return;
  if (0 == strcmp (Record->Type, "TBD") ||
      wxYES ==
      wxMessageBox ("Are you sure you want to delete this data item\n"
		    "and all of its sub-items (if any)?  If this is\n"
		    "an item originally present in the framework\n"
		    "(in other words, an item not added by you),\n"
		    "then removing it could introduce incompatibilities\n"
		    "with existing project datasets.", "Confirm",
		    wxCENTRE | wxYES_NO | wxICON_QUESTION))
    {
      LastSelected = -1;
      XmlChange (Selected, RootXml);
      *Pred = Record->Next;
      if (Record->Children != NULL)
	ClearXml (Record->Children);
      free (Record->Name);
      free (Record->Type);
      free (Record->Hint);
      free (Record->Text);
      free (Record);
      ItemTree->Delete (Selected);
      LastSelected = -1;
    }
}

//---------------------------------------------------------------------------
// A handler called when the "Add sub-item" button is clicked.

static const char DefaultHint[] =
  "This is a newly-added item, not present in the original framework.  "
  "Don\'t forget to change the title of the data item and the hint to something "
  "meaningful.  You should probably keep the value in the text-editor (i.e., \"TBD\" "
  "is a good default value).  The document-assignments, applicability, and control-"
  "category settings have been duplicated from the data item above, and should "
  "be changed if they\'re not what\'s needed.";

void
XmlEditorDialogClass::AddSubItem (wxCommandEvent & event)
{
  wxTreeItemId Selected = ItemTree->GetSelection ();
  struct XmlStruct *Record = FindXml (RootXml, Selected);
  if (Record == NULL)
    return;
  if (HintEditable)
    {
      struct XmlStruct *NewRecord;
      NewRecord = (struct XmlStruct *) malloc (sizeof (struct XmlStruct));
      if (NewRecord == NULL)
	{
	OutOfMemory:
	  wxMessageBox ("Out of memory.\nCannot add further items.\n"
			"Please save and exit.", "Fatal error",
			wxCENTRE | wxOK | wxICON_ERROR);
	  return;
	}
      LastSelected = Record->Id;
      XmlChange (Record->Id, RootXml);
      *NewRecord = *Record;
      NewRecord->Parent = Record;
      NewRecord->Next = Record->Children;
      NewRecord->Children = NULL;
      Record->Children = NewRecord;
      NewRecord->Name = (char *) malloc (4);
      if (NewRecord->Name == NULL)
	goto OutOfMemory;
      strcpy (NewRecord->Name, "TBD");
      NewRecord->Type = (char *) malloc (4);
      if (NewRecord->Type == NULL)
	goto OutOfMemory;
      strcpy (NewRecord->Type, "TBD");
      NewRecord->Text = (char *) malloc (4);
      if (NewRecord->Text == NULL)
	goto OutOfMemory;
      strcpy (NewRecord->Text, "TBD");
      NewRecord->Hint = (char *) malloc (sizeof (DefaultHint));
      if (NewRecord->Hint == NULL)
	goto OutOfMemory;
      strcpy (NewRecord->Hint, DefaultHint);
      NewRecord->Color = wxBLACK;
      NewRecord->Id = ItemTree->PrependItem (Record->Id, "TBD");
      ItemText->SetValue (NewRecord->Text);
      HintBox->SetValue (NewRecord->Hint);
      DatasetNewlyChanged = 1;
      LastSelected = NewRecord->Id;
      XmlChange (NewRecord->Id, RootXml);
      ItemTree->Expand (Record->Id);
      ItemTree->SelectItem (NewRecord->Id);
      Add->Enable (TRUE);
      AddSub->Enable (TRUE);
      Del->Enable (TRUE);
    }
  else
    {
      struct XmlStruct *NewRecord =
	CreateTreeCopy (AllowableAddSubs (Record));
      if (NewRecord == NULL)
	{
	  wxMessageBox ("Sorry, cannot create new item.", "Fatal error",
			wxCENTRE | wxOK | wxICON_ERROR);
	  return;
	}
      LastSelected = Record->Id;
      XmlChange (Record->Id, RootXml);
      NewRecord->Parent = Record;
      NewRecord->Next = Record->Children;
      //NewRecord->Children = NULL;
      Record->Children = NewRecord;
      NewRecord->Id = ItemTree->PrependItem (Record->Id, NewRecord->Name);
      FillSubTree (NewRecord);
      ItemText->SetValue (NewRecord->Text);
      HintBox->SetValue (NewRecord->Hint);
      DatasetNewlyChanged = 1;
      LastSelected = NewRecord->Id;
      XmlChange (NewRecord->Id, RootXml);
      ItemTree->Expand (Record->Id);
      ItemTree->SelectItem (NewRecord->Id);
      if (HintEditable)
	{
	  Add->Enable (TRUE);
	  AddSub->Enable (TRUE);
	}
      else
	{
	  Add->Enable (NULL != AllowableAddSiblings (NewRecord));
	  AddSub->Enable (NULL != AllowableAddSubs (NewRecord));
	}
      Del->Enable (TRUE);
    }
}

//---------------------------------------------------------------------------
// A handler called when the "Add sibling item" button is clicked.

void
XmlEditorDialogClass::AddItem (wxCommandEvent & event)
{
  wxTreeItemId Selected = ItemTree->GetSelection ();
  struct XmlStruct *Record = FindXml (RootXml, Selected);
  if (Record == NULL || Record == RootXml)
    return;
  if (HintEditable)
    {
      struct XmlStruct *NewRecord;
      NewRecord = (struct XmlStruct *) malloc (sizeof (struct XmlStruct));
      if (NewRecord == NULL)
	{
	OutOfMemory:
	  wxMessageBox ("Out of memory.\nCannot add further items.\n"
			"Please save and exit.", "Fatal error",
			wxCENTRE | wxOK | wxICON_ERROR);
	  return;
	}
      LastSelected = Record->Id;
      XmlChange (Record->Id, RootXml);
      *NewRecord = *Record;
      NewRecord->Parent = Record->Parent;
      NewRecord->Next = Record->Next;
      NewRecord->Children = NULL;
      Record->Next = NewRecord;
      NewRecord->Name = (char *) malloc (4);
      if (NewRecord->Name == NULL)
	goto OutOfMemory;
      strcpy (NewRecord->Name, "TBD");
      NewRecord->Type = (char *) malloc (4);
      if (NewRecord->Type == NULL)
	goto OutOfMemory;
      strcpy (NewRecord->Type, "TBD");
      NewRecord->Text = (char *) malloc (4);
      if (NewRecord->Text == NULL)
	goto OutOfMemory;
      strcpy (NewRecord->Text, "TBD");
      NewRecord->Hint = (char *) malloc (sizeof (DefaultHint));
      if (NewRecord->Hint == NULL)
	goto OutOfMemory;
      strcpy (NewRecord->Hint, DefaultHint);
      NewRecord->Color = wxBLACK;
      NewRecord->Id =
	ItemTree->InsertItem (NewRecord->Parent->Id, Record->Id, "TBD");
      ItemText->SetValue (NewRecord->Text);
      HintBox->SetValue (NewRecord->Hint);
      DatasetNewlyChanged = 1;
      LastSelected = NewRecord->Id;
      XmlChange (NewRecord->Id, RootXml);
      ItemTree->SelectItem (NewRecord->Id);
      Add->Enable (TRUE);
      AddSub->Enable (TRUE);
      Del->Enable (TRUE);
    }
  else
    {
      struct XmlStruct *NewRecord =
	CreateTreeCopy (AllowableAddSiblings (Record));
      if (NewRecord == NULL)
	{
	  wxMessageBox ("Sorry, cannot create the new item.", "Fatal error",
			wxCENTRE | wxOK | wxICON_ERROR);
	  return;
	}
      LastSelected = Record->Id;
      XmlChange (Record->Id, RootXml);
      NewRecord->Parent = Record->Parent;
      NewRecord->Next = Record->Next;
      Record->Next = NewRecord;
      NewRecord->Id =
	ItemTree->InsertItem (NewRecord->Parent->Id, Record->Id,
			      NewRecord->Name);
      FillSubTree (NewRecord);
      ItemText->SetValue (NewRecord->Text);
      HintBox->SetValue (NewRecord->Hint);
      DatasetNewlyChanged = 1;
      LastSelected = NewRecord->Id;
      XmlChange (NewRecord->Id, RootXml);
      ItemTree->SelectItem (NewRecord->Id);
      if (HintEditable)
	{
	  Add->Enable (TRUE);
	  AddSub->Enable (TRUE);
	}
      else
	{
	  Add->Enable (NULL != AllowableAddSiblings (NewRecord));
	  AddSub->Enable (NULL != AllowableAddSubs (NewRecord));
	}
      Del->Enable (TRUE);
    }
}

//---------------------------------------------------------------------------
// Given a pointer to a record in the template structure (or to any other
// XmlStruct record), copies the record and its entire subtree to memory,
// allocated space as required.  Returns a pointer to the copy of the subtree,
// or NULL on failure.  Operation is recursive.  On failure, all allocated
// memory is deallocated.

struct XmlStruct *
XmlEditorDialogClass::CreateTreeCopy (struct XmlStruct *Source)
{
  struct XmlStruct *Record, *SourceChild, *Child;
  //struct XmlStruct *LastChild;
  if (Source == NULL)
    return (NULL);
  // Allocate the new record, and duplicate the source record
  Record = (struct XmlStruct *) malloc (sizeof (struct XmlStruct));
  if (Record == NULL)
    return (NULL);
  *Record = *Source;
  Record->Name = (char *) malloc (strlen (Source->Name) + 1);
  if (Record->Name == NULL)
    {
      free (Record);
      return (NULL);
    }
  strcpy (Record->Name, Source->Name);
  Record->Type = (char *) malloc (strlen (Source->Type) + 1);
  if (Record->Type == NULL)
    {
      free (Record->Name);
      free (Record);
      return (NULL);
    }
  strcpy (Record->Type, Source->Type);
  Record->Hint = (char *) malloc (strlen (Source->Hint) + 1);
  if (Record->Hint == NULL)
    {
      free (Record->Type);
      free (Record->Name);
      free (Record);
      return (NULL);
    }
  strcpy (Record->Hint, Source->Hint);
  Record->Text = (char *) malloc (strlen (Source->Text) + 1);
  if (Record->Text == NULL)
    {
      free (Record->Hint);
      free (Record->Type);
      free (Record->Name);
      free (Record);
      return (NULL);
    }
  strcpy (Record->Text, Source->Text);
  Record->Parent = NULL;
  Record->Next = NULL;
  Record->Children = NULL;
  // Loop on the children of the source record.
  for (SourceChild = Source->Children; SourceChild != NULL;
       SourceChild = SourceChild->Next)
    {
      Child = CreateTreeCopy (SourceChild);
      if (Child == NULL)
	{
	  ClearXml (Record);
	  free (Record->Text);
	  free (Record->Hint);
	  free (Record->Type);
	  free (Record->Name);
	  free (Record);
	  return (NULL);
	}
      Child->Parent = Record;
      if (Record->Children == NULL)
	Record->Children = Child;
      //else
      //  LastChild->Next = Child;
      //LastChild = Child;
    }
  // All done!
  return (Record);
}

//---------------------------------------------------------------------------
// Given a pointer to an XmlStruct record which has just been added to the 
// ItemTree, adds the sub-tree to the ItemTree as well.  This function
// operates recursively.

void
XmlEditorDialogClass::FillSubTree (struct XmlStruct *Xml)
{
  struct XmlStruct *Next, *Last = NULL;
  for (Next = Xml->Children; Next != NULL; Next = Next->Next)
    {
      if (Next == Xml->Children)
	Next->Id = ItemTree->PrependItem (Xml->Id, Next->Name);
      else
	Next->Id =
	  ItemTree->InsertItem (Next->Parent->Id, Last->Id, Next->Name);
      if (Next->Children != NULL)
	FillSubTree (Next);
      Last = Next;
    }
}

//---------------------------------------------------------------------------
// Handlers called when the various control-category buttons are clicked.

void
XmlEditorDialogClass::ClickCcE (wxCommandEvent & event)
{
  CcChange (RootXml, DO178B_LEVEL_E, CcE);
}

void
XmlEditorDialogClass::ClickCcD (wxCommandEvent & event)
{
  CcChange (RootXml, DO178B_LEVEL_D, CcD);
}

void
XmlEditorDialogClass::ClickCcC (wxCommandEvent & event)
{
  CcChange (RootXml, DO178B_LEVEL_C, CcC);
}

void
XmlEditorDialogClass::ClickCcB (wxCommandEvent & event)
{
  CcChange (RootXml, DO178B_LEVEL_B, CcB);
}

void
XmlEditorDialogClass::ClickCcA (wxCommandEvent & event)
{
  CcChange (RootXml, DO178B_LEVEL_A, CcA);
}

//---------------------------------------------------------------------------
// Handlers called when the various "satisfaction" buttons are clicked.

void
XmlEditorDialogClass::ClickSatE (wxCommandEvent & event)
{
  SatChange (RootXml, DO178B_LEVEL_E, BitmapSatE);
}

void
XmlEditorDialogClass::ClickSatD (wxCommandEvent & event)
{
  SatChange (RootXml, DO178B_LEVEL_D, BitmapSatD);
}

void
XmlEditorDialogClass::ClickSatC (wxCommandEvent & event)
{
  SatChange (RootXml, DO178B_LEVEL_C, BitmapSatC);
}

void
XmlEditorDialogClass::ClickSatB (wxCommandEvent & event)
{
  SatChange (RootXml, DO178B_LEVEL_B, BitmapSatB);
}

void
XmlEditorDialogClass::ClickSatA (wxCommandEvent & event)
{
  SatChange (RootXml, DO178B_LEVEL_A, BitmapSatA);
}

//---------------------------------------------------------------------------
// Handlers that get called when document-type radio-buttons are clicked.

void
XmlEditorDialogClass::SqarClick (wxCommandEvent & event)
{

}

void
XmlEditorDialogClass::ScmrClick (wxCommandEvent & event)
{

}

void
XmlEditorDialogClass::SvrClick (wxCommandEvent & event)
{

}

void
XmlEditorDialogClass::SasClick (wxCommandEvent & event)
{
  DocumentChange (RootXml, SAS, SasCheck);
}

void
XmlEditorDialogClass::SciClick (wxCommandEvent & event)
{
  DocumentChange (RootXml, SCI, SciCheck);
}

void
XmlEditorDialogClass::SeciClick (wxCommandEvent & event)
{
  DocumentChange (RootXml, SECI, SeciCheck);
}

void
XmlEditorDialogClass::SvcpClick (wxCommandEvent & event)
{
  DocumentChange (RootXml, SVCP, SvcpCheck);
}

void
XmlEditorDialogClass::DdClick (wxCommandEvent & event)
{
  DocumentChange (RootXml, DD, DdCheck);
}

void
XmlEditorDialogClass::SrdClick (wxCommandEvent & event)
{
  DocumentChange (RootXml, SRD, SrdCheck);
}

void
XmlEditorDialogClass::ScsClick (wxCommandEvent & event)
{
  DocumentChange (RootXml, SCS, ScsCheck);
}

void
XmlEditorDialogClass::SdsClick (wxCommandEvent & event)
{
  DocumentChange (RootXml, SDS, SdsCheck);
}

void
XmlEditorDialogClass::SrsClick (wxCommandEvent & event)
{
  DocumentChange (RootXml, SRS, SrsCheck);
}

void
XmlEditorDialogClass::SqapClick (wxCommandEvent & event)
{
  DocumentChange (RootXml, SQAP, SqapCheck);
}

void
XmlEditorDialogClass::ScmpClick (wxCommandEvent & event)
{
  DocumentChange (RootXml, SCMP, ScmpCheck);
}

void
XmlEditorDialogClass::SvpClick (wxCommandEvent & event)
{
  DocumentChange (RootXml, SVP, SvpCheck);
}

void
XmlEditorDialogClass::SdpClick (wxCommandEvent & event)
{
  DocumentChange (RootXml, SDP, SdpCheck);
}

void
XmlEditorDialogClass::PsacClick (wxCommandEvent & event)
{
  DocumentChange (RootXml, PSAC, PsacCheck);
}

//---------------------------------------------------------------------------
// A handler called when outline is clicked.

void
XmlEditorDialogClass::ClickTree (wxTreeEvent & event)
{
  int Bypassing = ClickTreeBypass;
  wxTreeItemId Selected;
  // Default editor button setup.
  Add->Enable (HintEditable);
  AddSub->Enable (HintEditable);
  Del->Enable (HintEditable);
  if (ClickTreeBypass)
    {
      Selected = ItemTree->GetSelection ();
      ClickTreeBypass = 0;
    }
  else
    {
      Selected = event.GetItem ();
      if (Selected == RootXml->Id)
	{
	  Add->Enable (FALSE);
	  Del->Enable (FALSE);
	}
    }
  if (Bypassing || Selected != LastSelected)
    {
      if (Bypassing)
	LastSelected = Selected;
      // Here, need to write whatever was in the editor box to the OLD tree item.
      /*
         The rationale for the funky stuff below is probably bogus,
         because I now think the problems I experienced were due to 
         inadequate buffer sizes in XmlChange blowing the stack.  
         However, here's the possibly-bogus explanation:

         I don't know why (this is the only time I've seen it)
         but the most-significant word of "this" gets overwritten 
         on return from XmlChange at this point.  (It's perfectly okay
         INSIDE of XmlChange.)  Perhaps it's a PPC thing.  It means, though,
         that we have to capture "this" before using XmlChange, and then to
         explicitly use "that" throughout the remainder of the function, 
         rather than supposing that the references to class members will
         work properly.  Using "that->" everywhere is not only tedious and
         confusing, but also prone to error (from being forgotten), so I've
         split the ClickTree function into two parts:  ClickTree and ClickCleanup.
       */
      XmlEditorDialogClass *that = this;
      if (LastSelected != (wxTreeItemId) - 1)
	XmlChange (LastSelected, RootXml);

      // Now set up the display for the new item.
      that->LastSelected = Selected;
      XmlStruct *Record = FindXml (that->RootXml, Selected);
      that->ClickCleanup (Record);
    }
}

// ... continuation of ClickTree, to account for the "this"-corruption error.
// (Later:  I now believe there's no such bug, but the code is okay anyhow.)
// Basically, just sets up the on-screen display for whatever the newly
// selected data record contains.

void
XmlEditorDialogClass::ClickCleanup (struct XmlStruct *Record)
{
  FrameworkSet (Record);
  if (Record != NULL)
    {
      // Hint.
      if (Record->Hint != NULL)
	HintBox->SetValue (Record->Hint);
      else
	HintBox->SetValue ("");
      // Editor text.
      if (Record->Text != NULL)
	ItemText->SetValue (Record->Text);
      else
	ItemText->SetValue ("");
      // Set up the Add/Del/... buttons.
      if (Record->Parent == NULL)
	{
	  Add->Enable (FALSE);
	  AddSub->Enable (NULL != AllowableAddSubs (Record));
	  Del->Enable (FALSE);
	}
      else if (HintEditable)
	{
	  Add->Enable (TRUE);
	  AddSub->Enable (TRUE);
	  Del->Enable (TRUE);
	}
      else
	{
	  Add->Enable (NULL != AllowableAddSiblings (Record));
	  AddSub->Enable (NULL != AllowableAddSubs (Record));
	  Del->Enable (TRUE);
	}
      ItemText->Refresh ();
      HintBox->Refresh ();
    }
  else
    {
      ItemText->SetValue ("");
      HintBox->SetLabel ("");
      ItemText->Refresh ();
      HintBox->Refresh ();
    }
}

//---------------------------------------------------------------------------
// An auxiliary function (not an event-handler, though used by event handlers)
// that updates the in-memory hierarchical data structure, given changes in
// the displayed outline.

#define TYPE_FORMAT \
   "In addition to the visible title, which has been changed,\n" \
   "there is also an underlying invisible title, determining\n" \
   "what types of data items can legally be inserted at this position.\n" \
   "If this is a NEW data item, the two titles should be the same.\n" \
   "But if you are merely changing the title of an OLD data item,\n" \
   "changing the underlying title could make your old datasets\n" \
   "incompatible!  Should I change the underlying title?\n" \
   "    Underlying title was:  \"%s\"\n" \
   "    New underlying title:  \"%s\""

void
XmlEditorDialogClass::XmlChange (wxTreeItemId Selected, struct XmlStruct *Xml)
{
  XmlStruct *Record = FindXml (Xml, Selected);
  if (Record == NULL)
    return;

  // Make sure all of the fields we need to check have been allocated.
  if (Record->Text == NULL)
    {
      Record->Text = (char *) malloc (1 + strlen (""));
      if (Record->Text == NULL)
	{
	OutOfMemory:
	  wxMessageBox ("Out of memory.\nCannot retain further changes\n"
			"Please save and exit.", "Fatal error",
			wxCENTRE | wxOK | wxICON_ERROR);
	  return;
	}
      strcpy (Record->Text, "");
    }
  if (Record->Hint == NULL)
    {
      Record->Hint = (char *) malloc (1 + strlen (""));
      if (Record->Hint == NULL)
	goto OutOfMemory;
      strcpy (Record->Hint, "");
    }

  // Save any changes from the editor window.
  wxString Text = ItemText->GetValue ();
  if (0 != strcmp (Record->Text, Text))
    {
      DatasetNewlyChanged = 1;
      free (Record->Text);
      Record->Text = (char *) malloc (Text.Len () + 1);
      if (Record->Text == NULL)
	goto OutOfMemory;
      strcpy (Record->Text, Text);
    }

  // Save any changes to the data-item name.
  Text = ItemTree->GetItemText (Selected);
  if (0 != strcmp (Record->Name, Text))
    {
      DatasetNewlyChanged = 1;
      free (Record->Name);
      Record->Name = (char *) malloc (Text.Len () + 1);
      if (Record->Name == NULL)
	goto OutOfMemory;
      strcpy (Record->Name, Text);
    }

  // If we are in framework-modification mode, the same change to the name
  // must also apply to the type.
  if (HintEditable)
    {
      void NameToTag (const char *Name, char *Tag);
      char Tag[513], Warning[strlen (TYPE_FORMAT) + sizeof (Tag) / 3];
      Text = ItemTree->GetItemText (Selected);
      // The following check is to prevent buffer overflows in the 
      // prompt and the tag.  The worst possible expansion in a tag
      // is if EVERY character is converted to something like &#n;
      // which would be a maximum expansion of 6-to-1.
      if (Text.Len () < (sizeof (Tag) - 1) / 6)
	{
	  NameToTag (Text, Tag);
	  if (0 != strcmp (Record->Type, Tag))
	    {
	      sprintf (Warning, TYPE_FORMAT, Record->Type, Tag);
	      if (0 == strcmp (Record->Type, "")
		  || 0 == strcmp (Record->Type, "TBD")
		  || wxYES == wxMessageBox (Warning, "Compatibility danger!",
					    wxYES_NO | wxCENTRE |
					    wxICON_EXCLAMATION))
		{
		  DatasetNewlyChanged = 1;
		  free (Record->Type);
		  Record->Type = (char *) malloc (strlen (Tag) + 1);
		  if (Record->Type == NULL)
		    goto OutOfMemory;
		  strcpy (Record->Type, Tag);
		}
	    }
	}
    }

  // Save any changes to the hint window.
  Text = HintBox->GetValue ();
  if (0 != strcmp (Record->Hint, Text))
    {
      DatasetNewlyChanged = 1;
      free (Record->Hint);
      Record->Hint = (char *) malloc (Text.Len () + 1);
      if (Record->Hint == NULL)
	goto OutOfMemory;
      strcpy (Record->Hint, Text);
    }

  // Note that any framework changes (the relevant-document list, 
  // applicability settings, the control category settings) will 
  // already have been made automatically, since they are associated
  // with events attached to check-boxes and buttons.  

  // Visually mark this item, and all ancestors, as changed.
  if (DatasetNewlyChanged)
    {
      DatasetChanged = 1;
      DatasetNewlyChanged = 0;
      XmlStruct *Parent;
      for (Parent = Record; Parent != NULL; Parent = Parent->Parent)
	{
	  Parent->Color = CHANGE_COLOR;
	  ItemTree->SetItemTextColour (Parent->Id, *CHANGE_COLOR);
	}
    }
}

//---------------------------------------------------------------------------
// An auxiliary function (not a handler, though used by handlers) that can
// locate the predecessor (either the prior sibling or else the parent)
// of any given data item.  That predecessor, and no other data item, 
// contains a pointer back to the data item we are finding the predecessor
// of.  This function returns a pointer to that pointer (which will be
// either the Next or Children field of the predecessor data item).
// If this fails, which can happen only if we started from the root element,
// NULL is returned.

struct XmlStruct **
XmlEditorDialogClass::FindPredecessor (struct XmlStruct *Xml,
				       struct XmlStruct *Record)
{
  struct XmlStruct *Pred;
  if (Record == NULL || Record == Xml)
    return (NULL);
  // We can always find the predecessor by moving back to the
  // Parent, and then looping on the Parent's Children.
  // We have to be a little careful with this algorithm, since
  // we don't ever want to be fooled into returning &Pred.
  if (Record->Parent->Children == Record)
    {
      // In this case the record was the FIRST child of its parent.
      return (&(Record->Parent->Children));
    }
  for (Pred = Record->Parent->Children; Pred->Next != Record;
       Pred = Pred->Next);
  return (&(Pred->Next));
}

//---------------------------------------------------------------------------
// An auxiliary function (not a handler, though used by handlers) that 
// sets all of the labels for a bitmap button to the same bitmap.

void
XmlEditorDialogClass::ButtonBitmap (wxBitmapButton * Button, wxBitmap Bitmap)
{
  Button->SetBitmapLabel (Bitmap);
  Button->SetBitmapFocus (Bitmap);
  Button->SetBitmapDisabled (Bitmap);
  Button->SetBitmapSelected (Bitmap);
}

//---------------------------------------------------------------------------
// An auxiliary function (not a handler, though used by handlers) this 
// changes the in-memory hierarchical data structure according to an 
// action on a CC button.

void
XmlEditorDialogClass::CcChange (struct XmlStruct *Xml, int Level,
				wxBitmapButton * Button)
{
  int i;
  wxTreeItemId Selected = ItemTree->GetSelection ();
  struct XmlStruct *Record = FindXml (Xml, Selected);
  if (Record == NULL)
    return;

  DatasetNewlyChanged = 1;
  i = Record->Applicability[Level] & CC_MASK;
  i += 16;
  if (i != CC_DISCRETION && i != CC_1 && i != CC_2)
    i = 0;
  Record->Applicability[Level] =
    i | (Record->Applicability[Level] & ~CC_MASK);
  ButtonBitmap (Button, (CcFunc (i >> 4)));
  XmlChange (Selected, Xml);
}

//---------------------------------------------------------------------------
// An auxiliary function (not a handler, though used by handlers) that 
// changes the in-memory hierarchical data structure according to an 
// action on a Sat button.

void
XmlEditorDialogClass::SatChange (struct XmlStruct *Xml, int Level,
				 wxBitmapButton * Button)
{
  int i;
  wxTreeItemId Selected = ItemTree->GetSelection ();
  struct XmlStruct *Record = FindXml (Xml, Selected);
  if (Record == NULL)
    return;

  DatasetNewlyChanged = 1;
  i = Record->Applicability[Level] & SATISFIED_MASK;
  i++;
  if (i != SATISFIED_DISCRETION && i != SATISFIED_YES
      && i != SATISFIED_INDEPENDENCE)
    i = 0;
  Record->Applicability[Level] =
    i | (Record->Applicability[Level] & ~SATISFIED_MASK);
  ButtonBitmap (Button, SatisfactionFunc (i));
  XmlChange (Selected, Xml);
}

//---------------------------------------------------------------------------
// An auxiliary function (not a handler, though used by handlers) that 
// looks at one of the document check-boxes, and updates the in-memory
// hierarchical data structure accordingly.

void
XmlEditorDialogClass::DocumentChange (struct XmlStruct *Xml, int DocNum,
				      wxCheckBox * Box)
{
  wxTreeItemId Selected = ItemTree->GetSelection ();
  struct XmlStruct *Record = FindXml (Xml, Selected);
  if (Record == NULL)
    return;

  DatasetNewlyChanged = 1;
  Record->Documents[DocNum] = Box->GetValue ();
  XmlChange (Selected, Xml);
}

//---------------------------------------------------------------------------
// An auxiliary function (not a handler, though used by handlers) that 
// descends recursively through all levels of the in-memory hierarchical data
// structure, to set the "by section" document dependencies.
// On first entry, all of the section boxes must be unchecked.
// In other words, it detects which documents are affected by a given data
// item OR any of its descendents.

void
XmlEditorDialogClass::SetSectionDocs (struct XmlStruct *Record)
{
  PsacSection->SetValue (Record->Documents[PSAC] || PsacSection->GetValue ());
  SdpSection->SetValue (Record->Documents[SDP] || SdpSection->GetValue ());
  SvpSection->SetValue (Record->Documents[SVP] || SvpSection->GetValue ());
  ScmpSection->SetValue (Record->Documents[SCMP] || ScmpSection->GetValue ());
  SqapSection->SetValue (Record->Documents[SQAP] || SqapSection->GetValue ());
  SrsSection->SetValue (Record->Documents[SRS] || SrsSection->GetValue ());
  SdsSection->SetValue (Record->Documents[SDS] || SdsSection->GetValue ());
  ScsSection->SetValue (Record->Documents[SCS] || ScsSection->GetValue ());
  SrdSection->SetValue (Record->Documents[SRD] || SrdSection->GetValue ());
  DdSection->SetValue (Record->Documents[DD] || DdSection->GetValue ());
  SvcpSection->SetValue (Record->Documents[SVCP] || SvcpSection->GetValue ());
  SeciSection->SetValue (Record->Documents[SECI] || SeciSection->GetValue ());
  SciSection->SetValue (Record->Documents[SCI] || SciSection->GetValue ());
  SasSection->SetValue (Record->Documents[SAS] || SasSection->GetValue ());
  for (Record = Record->Children; Record != NULL; Record = Record->Next)
    SetSectionDocs (Record);
}

//---------------------------------------------------------------------------
// An auxiliary function (not a handler, though used by handlers) that can 
// set up the CC, Sat, and Document boxes appropriately for any given data
// item.

void
XmlEditorDialogClass::FrameworkSet (struct XmlStruct *Record)
{
  PsacSection->SetValue (FALSE);
  SdpSection->SetValue (FALSE);
  SvpSection->SetValue (FALSE);
  ScmpSection->SetValue (FALSE);
  SqapSection->SetValue (FALSE);
  SrsSection->SetValue (FALSE);
  SdsSection->SetValue (FALSE);
  ScsSection->SetValue (FALSE);
  SrdSection->SetValue (FALSE);
  DdSection->SetValue (FALSE);
  SvcpSection->SetValue (FALSE);
  SeciSection->SetValue (FALSE);
  SciSection->SetValue (FALSE);
  SasSection->SetValue (FALSE);
  if (Record == NULL)
    {
      PsacCheck->SetValue (FALSE);
      SdpCheck->SetValue (FALSE);
      SvpCheck->SetValue (FALSE);
      ScmpCheck->SetValue (FALSE);
      SqapCheck->SetValue (FALSE);
      SrsCheck->SetValue (FALSE);
      SdsCheck->SetValue (FALSE);
      ScsCheck->SetValue (FALSE);
      SrdCheck->SetValue (FALSE);
      DdCheck->SetValue (FALSE);
      SvcpCheck->SetValue (FALSE);
      SeciCheck->SetValue (FALSE);
      SciCheck->SetValue (FALSE);
      SasCheck->SetValue (FALSE);
      ButtonBitmap (BitmapSatA, SatisfactionFunc (0));
      ButtonBitmap (BitmapSatB, SatisfactionFunc (0));
      ButtonBitmap (BitmapSatC, SatisfactionFunc (0));
      ButtonBitmap (BitmapSatD, SatisfactionFunc (0));
      ButtonBitmap (BitmapSatE, SatisfactionFunc (0));
      ButtonBitmap (CcA, CcFunc (0));
      ButtonBitmap (CcB, CcFunc (0));
      ButtonBitmap (CcC, CcFunc (0));
      ButtonBitmap (CcD, CcFunc (0));
      ButtonBitmap (CcE, CcFunc (0));
    }
  else
    {
      PsacCheck->SetValue (Record->Documents[PSAC]);
      SdpCheck->SetValue (Record->Documents[SDP]);
      SvpCheck->SetValue (Record->Documents[SVP]);
      ScmpCheck->SetValue (Record->Documents[SCMP]);
      SqapCheck->SetValue (Record->Documents[SQAP]);
      SrsCheck->SetValue (Record->Documents[SRS]);
      SdsCheck->SetValue (Record->Documents[SDS]);
      ScsCheck->SetValue (Record->Documents[SCS]);
      SrdCheck->SetValue (Record->Documents[SRD]);
      DdCheck->SetValue (Record->Documents[DD]);
      SvcpCheck->SetValue (Record->Documents[SVCP]);
      SeciCheck->SetValue (Record->Documents[SECI]);
      SciCheck->SetValue (Record->Documents[SCI]);
      SasCheck->SetValue (Record->Documents[SAS]);
      SetSectionDocs (Record);
      ButtonBitmap (BitmapSatA,
		    SatisfactionFunc (Record->Applicability[DO178B_LEVEL_A] &
				      SATISFIED_MASK));
      ButtonBitmap (BitmapSatB,
		    SatisfactionFunc (Record->Applicability[DO178B_LEVEL_B] &
				      SATISFIED_MASK));
      ButtonBitmap (BitmapSatC,
		    SatisfactionFunc (Record->Applicability[DO178B_LEVEL_C] &
				      SATISFIED_MASK));
      ButtonBitmap (BitmapSatD,
		    SatisfactionFunc (Record->Applicability[DO178B_LEVEL_D] &
				      SATISFIED_MASK));
      ButtonBitmap (BitmapSatE,
		    SatisfactionFunc (Record->Applicability[DO178B_LEVEL_E] &
				      SATISFIED_MASK));
      ButtonBitmap (CcA,
		    CcFunc ((Record->Applicability[DO178B_LEVEL_A] & CC_MASK)
			    >> 4));
      ButtonBitmap (CcB,
		    CcFunc ((Record->Applicability[DO178B_LEVEL_B] & CC_MASK)
			    >> 4));
      ButtonBitmap (CcC,
		    CcFunc ((Record->Applicability[DO178B_LEVEL_C] & CC_MASK)
			    >> 4));
      ButtonBitmap (CcD,
		    CcFunc ((Record->Applicability[DO178B_LEVEL_D] & CC_MASK)
			    >> 4));
      ButtonBitmap (CcE,
		    CcFunc ((Record->Applicability[DO178B_LEVEL_E] & CC_MASK)
			    >> 4));
    }
  // The following stuff shouldn't be necessary -- and isn't in Linux -- but in Win32
  // the buttons don't get refreshed for some reason.  
  BitmapSatA->Refresh ();
  BitmapSatB->Refresh ();
  BitmapSatC->Refresh ();
  BitmapSatD->Refresh ();
  BitmapSatE->Refresh ();
  CcA->Refresh ();
  CcB->Refresh ();
  CcC->Refresh ();
  CcD->Refresh ();
  CcE->Refresh ();  
}

//--------------------------------------------------------------------------
// Given a pointer to an XmlStruct record, descends through the complete
// data structure and the template structure, comparing records, until it
// finds the template record associated with the specified data record.
// Returns NULL on failure.  The item needs to be found at the correct
// level of the hierarchy, with the correct ancestors, but the ordering
// of the siblings is ignored.

#define MAX_DEPTH 32
struct XmlStruct *
XmlEditorDialogClass::SeekTemplate (struct XmlStruct *Record)
{
  struct XmlStruct *Trace[MAX_DEPTH];
  struct XmlStruct *d;
  int Depth;
  if (Template == NULL || Template->Children == NULL)
    return (NULL);
  // In the Trace array, we store a trail of Record's ancestors up to 
  // the root of the data tree;
  for (Depth = 0, d = Record; d != NULL; d = d->Parent)
    Trace[Depth++] = d;
  // Now descend through the Template, looking for the records corresponding
  // to these ancestors.
  Depth -= 2;			// Trace[Depth] == RootXml->Children.
  for (d = Template->Children; Depth >= 0; Depth--)
    {
      for (; d != NULL; d = d->Next)
	if (!strcmp (d->Type, Trace[Depth]->Type))
	  break;
      if (d == NULL)
	return (NULL);
      if (Depth == 0)
	return (d);
      d = d->Children;
    }
  return (NULL);
}

//--------------------------------------------------------------------------

struct XmlStruct *
XmlEditorDialogClass::AllowableAddSiblings (struct XmlStruct *Record)
{
  // Find the associated template record.
  struct XmlStruct *t = SeekTemplate (Record);
  if (t == NULL)
    return (NULL);
  // A sibling can be added under a variety of circumstances, and these
  // have to be arranged in a priority structure, to insure that certain
  // possibilities aren't locked out permanently when multiple possibilities
  // exist.  Here are the cases, in order of preference:
  //    1.  A record of the SAME TYPE as the template sibling does not follow
  //        the record.  This is actually two subcases:  NO sibling follows
  //        the record, or else a sibling that doesn't match follows.
  //        In either case, the template sibling can be added.
  //    2.  The template sibling is of type "TBD ..." and doesn't match
  //        the sibling of the current record.  Then the template sibling
  //        can be added.  As it happens, this is included in case 1.
  //    3.  The current/template records are of type "TBD ...".  Then the
  //        template item (not its sibling) can be added.
  //    4.  The template sibling is of type "TBD ...".  Then the template
  //        sibling can be added, irrespective of the current sibling.
  if (t->Next != NULL)
    {
      if (Record->Next == NULL)
	return (t->Next);
      if (strcmp (t->Next->Type, Record->Next->Type))
	return (t->Next);
    }
  if (!strncmp (t->Type, "TBD", 3))
    return (t);
  if (t->Next != NULL && !strncmp (t->Next->Type, "TBD", 3))
    return (t->Next);
  return (NULL);
}

//--------------------------------------------------------------------------

struct XmlStruct *
XmlEditorDialogClass::AllowableAddSubs (struct XmlStruct *Record)
{
  // Find the associated template record.
  struct XmlStruct *t = SeekTemplate (Record);
  if (t == NULL)
    return (NULL);
  // The only allowed children which can be added correspond to the 
  // first child of the template record.  This child can be added
  // EITHER if it is missing from the data structure, OR if its
  // type begins with TBD.
  t = t->Children;
  if (t == NULL)
    return (NULL);
  if (Record->Children == NULL)
    return (t);
  if (!strncmp (t->Type, "TBD", 3))
    return (t);
  if (strcmp (t->Type, Record->Children->Type))
    return (t);
  return (NULL);
}
